package de.bsvrz.buv.plugin.bmvew;

import java.util.LinkedList;
import java.util.List;

/**
 * Speichert bisher empfangene Betriebsmeldungen. <br>
 * Hinweis: Diese Klasse ist als Singleton ausgelegt, damit auch zu einem spteren Zeitpunkt, nach dem schlieen des
 * Dialogs {@link MeldungsanzeigeDialog}, alle bereits empfangenen Betriebsmeldungen wieder angezeigt werden knnen.
 * 
 * @version $Revision: 1.2 $ / $Date: 2010/02/02 15:40:41 $ / ($Author: HofmannJ $)
 * @author HofmannJ
 * 
 */
public class MeldungenSpeicher
{
  /**
   * Verbindung zum DaV
   */
  DaVKommunikation _daVKommunikation = null;

  /**
   * Speichert die einzigste Instanz der Klasse {@link MeldungenSpeicher}.
   */
  private static MeldungenSpeicher _instanz = null;

  /**
   * Gibt die Maximale Anzahl von Betriebsmeldungen an.
   */
  private int _maximaleAnzahlBetriebsmeldungen = 1000;

  /**
   * Speichert die empfangenen Betriebsmeldungen und hlt diese vorrtig.
   */
  private List<Meldung> _lBetriebsmeldungen = null;

  /**
   * Liefert die einzigste Instanz der Klasse {@link MeldungenSpeicher}.
   * 
   * @return Einzigste Instanz der Klasse {@link MeldungenSpeicher}.
   */
  public static MeldungenSpeicher getInstanz()
  {
    if (_instanz == null)
    {
      _instanz = new MeldungenSpeicher();
    }
    return _instanz;
  }

  /**
   * Liefert die einzigste Instanz der Klasse {@link MeldungenSpeicher}.
   * 
   * @param maxAnzahlBetriebsmeldungen
   *          Setzt die maximale Anzahl von Betriebsmeldungen, die gespeichert werden. Wird nur bei der ersten
   *          Initialisierung der Singletonklasse gesetzt und ist im Nachhinein (z.B. beim zweiten Aufruf dieser
   *          Methode) nicht vernderbar. Defaultwert == 1000.
   * 
   * @return Einzigste Instanz der Klasse {@link MeldungenSpeicher}.
   */
  public static MeldungenSpeicher getInstanz(int maxAnzahlBetriebsmeldungen)
  {
    if (_instanz == null)
    {
      _instanz = new MeldungenSpeicher(maxAnzahlBetriebsmeldungen);
    }
    return _instanz;
  }

  /**
   * Privater, nicht ffentlicher Konstruktor dieser Singletonklasse.
   */
  private MeldungenSpeicher()
  {
    init();
  }

  /**
   * Privater, nicht ffentlicher Konstruktor dieser Singletonklasse.
   * 
   * @param maxAnzahlBetriebsmeldungen
   *          Setzt die maximale Anzahl von Betriebsmeldungen, die gespeichert werden.
   */
  private MeldungenSpeicher(int maxAnzahlBetriebsmeldungen)
  {
    _maximaleAnzahlBetriebsmeldungen = maxAnzahlBetriebsmeldungen;
    init();
  }

  /**
   * Fhrt die Initialisierung durch.
   * 
   */
  private void init()
  {
    _daVKommunikation = DaVKommunikation.getInstanz();

    _lBetriebsmeldungen = new LinkedList<Meldung>();

    IBetriebsmeldungListener betriebsmeldungListener = new IBetriebsmeldungListener() {

      public void betriebsmeldungEingang(BetriebsmeldungEreignis e)
      {
        /*
         * Abarbeiten der eingehenden Meldung.
         */
        List<Meldung> meldungen = e.getBetriebsmeldungen();
        for (Meldung meldung : meldungen)
        {
          hinzufuegenBetriebsmeldung(meldung);

        }

      }

    };

    _daVKommunikation.hinzufuegenBetriebsmeldungsListener(betriebsmeldungListener);

  }

  /**
   * Fgt der Liste der Betriebsmeldungen eine neue Betriebsmeldung am Anfang (Index 0) hinzu. Nachfolgende, "ltere"
   * Elemente werden um eine Position nach hinten verschoben. <br>
   * bersteigt beim Hinzufgen einer Betriebsmeldung die Anzahl der Betriebsmeldungen die maximale Anzahl von
   * Betriebsmeldungen, so wird die jeweils lteste (am frhsten hinzugefgte) Betriebsmeldung gelscht.
   * 
   * @param betriebsmeldung
   *          Betriebsmeldung (Instanz der Klasse {@link Meldung}), die hinzugefgt werden soll.
   */
  private void hinzufuegenBetriebsmeldung(Meldung betriebsmeldung)
  {
    _lBetriebsmeldungen.add(0, betriebsmeldung);

    /*
     * Prfen, ob die Anzahl der Betriebsmeldungen die Anzahl der maximal zulssigen Betriebsmeldungen bersteigt.
     */
    if (_lBetriebsmeldungen.size() > _maximaleAnzahlBetriebsmeldungen)
    {
      removeAeltesteBetriebsmeldung();
    }
  }

  /**
   * Entfernt die lteste Betriebsmeldung in der Liste der Betriebstmeldungen.
   * 
   */
  private void removeAeltesteBetriebsmeldung()
  {
    int letzteElementPosition = _lBetriebsmeldungen.size() - 1;
    _lBetriebsmeldungen.remove(letzteElementPosition);

  }

  /**
   * Liefert die Liste mit allen bisher empfangenen Betriebsmeldungen. Fr eine erneute Darstellung muss lediglich die
   * Tabelle der Betriebsmeldungen in {@link MeldungsanzeigeDialog} mit dieser Liste neu aufgebaut werden.
   * 
   * @return Liste mit allen empfangenen Betriebsmeldungen.
   */
  public List<Meldung> getBetriebsmeldungen()
  {
    return _lBetriebsmeldungen;
  }

  public void setBetriebsmeldungen(List<Meldung> meldungen)
  {
    _lBetriebsmeldungen = meldungen;
  }

}
