/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.berechtigung;

import java.util.EventObject;
import java.util.List;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Die Klasse stellt den passenden Event zum OberflaechenBerechtigungListener zur Verfgung. Mit ihm knnen
 * "Oberflchenelement" darber informiert werden, ob sie nun gesperrt oder freigegeben sind. Elemente, die diese
 * Information verarbeiten, mssen sich dem entsprechend ein- oder ausgrauen.
 * 
 * @version $Revision: 1.2 $ / $Date: 2009/12/08 08:50:11 $ / ($Author: HofmannJ $)
 * 
 * @author HofmannJ
 * 
 */
public class OberflaechenBerechtigungEreignis extends EventObject
{
  /**
   * Bei <code>true</code> erfolgt die Freigabe, bei <code>false</code> nicht.
   */
  private boolean _freigabe = true;

  /**
   * Liste von Systemobjekten, auf die sich dieser Event bezieht und fr die eine Freigabe erfolgt.
   */
  private List<SystemObject> _systemObjekteFreigabe = null;

  /**
   * Liste von Systemobjekten, auf die sich dieser Event bezieht und fr die eine Sperrung erfolgt.
   */
  private List<SystemObject> _systemObjekteSperrung = null;

  /**
   * Liste von Oberflaechenfunktionen, auf die sich dieser Event bezieht und fr die eine Freigabe erfolgt.
   */
  private List<OberflaechenFunktion> _oberflaechenFunktionFreigabe = null;

  /**
   * Liste von Oberflaechenfunktionen, auf die sich dieser Event bezieht und fr die eine Sperrung erfolgt.
   */
  private List<OberflaechenFunktion> _oberflaechenFunktionSperrung = null;

  /**
   * Konstruktor des Events. Im Objekt <code>source</code> wird das Objekt, auf welches sich der Event bezieht,
   * bergeben.
   * 
   * @param source
   *          Objekt, auf dass sich der Event bezieht. Es kann sich hierbei z.B. um eine Liste von Systemobjekten
   *          handeln.
   * @param freigabe
   *          Freigabe erfolgt wenn <code>true</code>, bei <code>false</code> erfolgt die Sperrung.
   */
  public OberflaechenBerechtigungEreignis(Object source, boolean freigabe, List<SystemObject> systemObjekteFreigabe,
      List<SystemObject> systemObjekteSperrung, List<OberflaechenFunktion> oberflaechenFunktionFreigabe,
      List<OberflaechenFunktion> oberflaechenFunktionSperrung)
  {
    super(source);
    this._freigabe = freigabe;

    this._systemObjekteFreigabe = systemObjekteFreigabe;
    this._systemObjekteSperrung = systemObjekteSperrung;
    this._oberflaechenFunktionFreigabe = oberflaechenFunktionFreigabe;
    this._oberflaechenFunktionSperrung = oberflaechenFunktionSperrung;
  }

  /**
   * Diese Methode prft, ob die Freigabe bereits erfolgte oder nicht.
   * 
   * @return Liefert <code>true</code>, wenn die Freigabe bereits erfolgte, sonst <code>false</code>.
   */
  public boolean hatFreigabe()
  {
    return _freigabe;
  }

  /**
   * Liefert eine Liste von Systemobjekten zurck, auf die sich dieser Event bezieht. <br>
   * Es wird die Liste der SystemObjekte zurckgeliefert, fr die eine Freigabe erfolgen kann.
   * 
   * @return Liste der Systemobjekte, auf die sich die Freigabe bezieht.
   */
  public List<SystemObject> getSystemObjekteFreigabe()
  {
    return _systemObjekteFreigabe;
  }

  /**
   * Liefert eine Liste von Systemobjekten zurck, auf die sich dieser Event bezieht. <br>
   * Es wird die Liste der SystemObjekte zurckgeliefert, fr die eine Sperrung erfolgt.
   * 
   * @return Liste der SystemObjekte, auf die sich die Sperrung bezieht.
   */
  public List<SystemObject> getSystemObjekteSperrung()
  {
    return _systemObjekteSperrung;
  }

  /**
   * Liefert eine Liste von Oberflaechenfunktionen zurck, auf die sich dieser Event bezieht. <br>
   * Es wird die Liste der Oberflaechenfunktionen zurckgeliefert, fr die eine Freigabe erfolgt.
   * 
   * @return Liste der SystemObjekte, auf die sich die Sperrung bezieht.
   */
  public List<OberflaechenFunktion> getOberflaechenFunktionFreigabe()
  {
    return _oberflaechenFunktionFreigabe;
  }

  /**
   * Liefert eine Liste von Oberflaechenfunktionen zurck, auf die sich dieser Event bezieht. <br>
   * Es wird die Liste der Oberflaechenfunktionen zurckgeliefert, fr die eine Sperrung erfolgt.
   * 
   * @return Liste der SystemObjekte, auf die sich die Sperrung bezieht.
   */
  public List<OberflaechenFunktion> getOberflaechenFunktionSperrung()
  {
    return _oberflaechenFunktionSperrung;
  }

  /**
   * Generierte serielle Versions UID dieses Events.
   */
  private static final long serialVersionUID = -247745193727961182L;

}
