/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.berechtigung;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Die Klasse implementiert einen binaeren Suchbaum. Diese Klase wird verwendet
 * um den Parameter des Berechtigungskonzeopts als Baumstruktur abzubilden.
 * 
 * @author Pittner
 * 
 */
public class SuchBaum
{

	/**
	 * Bezeichner fr die Traversirungsstrategie
	 */
	public static final int INORDER = 1;

	/**
	 * Bezeichner fr die Traversirungsstrategie
	 */
	public static final int PREORDER = 2;

	/**
	 * Bezeichner fr die Traversirungsstrategie
	 */
	public static final int POSTORDER = 3;

	/**
	 * Bezeichner fr die Traversirungsstrategie
	 */
	public static final int LEVELORDER = 4;

	/**
	 * Definition der Wurzel und des Pseudoknotens
	 */
	private BaumKnoten kopf, nullKnoten;

	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	/**
	 * Knotenklasse der Baumstruktur, als innere Klasse implementiert.
	 * 
	 * @author Pittner
	 * 
	 */
	static class BaumKnoten
	{

		Object schluessel;

		BaumKnoten links = null;

		BaumKnoten rechts = null;

		public BaumKnoten(Object e)
		{
			schluessel = e;
		}

		public BaumKnoten getLinks()
		{
			return links;
		}

		public BaumKnoten getRechts()
		{
			return rechts;
		}

		public Object getSchluessel()
		{
			return schluessel;
		}

		public void setLinks(BaumKnoten n)
		{
			links = n;
		}

		public void setRechts(BaumKnoten n)
		{
			rechts = n;
		}

		public int bestimmeEbene(Data d)
		{

			String text = d.getItem("Ebene").asTextValue().getText();

			Integer i = new Integer(text);

			return i;

		}

		public String toString()
		{
			return schluessel.toString();
		}

		public void printKnoten(StringBuffer sb, int depth, BaumKnoten nullNode)
		{
			for (int i = 0; i < depth; i++)
				sb.append("  ");
			sb.append(toString() + "\n");

			if (links != nullNode)
				links.printKnoten(sb, depth + 1, nullNode);
			if (rechts != nullNode)
				rechts.printKnoten(sb, depth + 1, nullNode);

		}
	}

	/**
	 * Konstruktor
	 */
	public SuchBaum()
	{
		kopf = new BaumKnoten(null);
		nullKnoten = new BaumKnoten(null);
		nullKnoten.setLinks(nullKnoten);
		nullKnoten.setRechts(nullKnoten);
		kopf.setRechts(nullKnoten);
	}

	/**
	 * Einfuegeoperation, welche ausgehend vom bergebenen Knoten als Startpunkt
	 * den aus den eingefeugten Knoten zurueckgibt
	 * 
	 * @param knoten
	 *          Startpunkt, von dem aus eine Einfgepositions gesucht wird
	 * @param d
	 *          das Datum des Knotens
	 * @return Knoten welcher eingefgt wurde
	 */
	public BaumKnoten einfuegen(BaumKnoten knoten, Data d)
	{

		if (knoten == null)
			knoten = kopf;

		BaumKnoten eltern = knoten;

		BaumKnoten kind = knoten.getRechts();

		while (kind != nullKnoten)
		{

			eltern = kind;

			int e = kind.bestimmeEbene(d);

			if (e > 0)
			{
				kind = eltern.getLinks();

			}
			else
			{
				kind = eltern.getRechts();
			}

		}

		BaumKnoten node = new BaumKnoten(d);
		node.setLinks(nullKnoten);
		node.setRechts(nullKnoten);

		int e = eltern.bestimmeEbene(d);

		if (e > 0)
		{
			eltern.setLinks(node);
		}
		else
		{
			eltern.setRechts(node);
		}

		return node;
	}

	/**
	 * Sucht iterativ das bergebene Datum in der Baumstruktur
	 * 
	 * @param d
	 *          das Datum, nach welchem gesucht wird
	 * @return true, wenn ein Knoten gefunden wurde
	 */
	public boolean finde(Data d)
	{
		return (findeKnoten(d) != null);
	}

	private BaumKnoten findeKnoten(Data d)
	{
		BaumKnoten n = kopf.getRechts();
		while (n != nullKnoten)
		{

			int e = n.bestimmeEbene(d);

			if (n.getSchluessel().equals(d))
				return n;

			if (e > 0)
				n.getLinks();
			else
				n.getRechts();
		}
		return n;
	}

	private BaumKnoten rekursivFindeKnoten(BaumKnoten n, Data d)
	{
		if (n != nullKnoten)
		{

			int e = n.bestimmeEbene(d);

			if (n.getSchluessel().equals(d))
				return n;

			if (e > 0)
				return rekursivFindeKnoten(n.getLinks(), d);
			else
				return rekursivFindeKnoten(n.getRechts(), d);
		}
		else
			return null;
	}

	/**
	 * Sucht rekursiv das bergebene Datum in der Baumstruktur
	 * 
	 * @param d
	 *          das Datum, nach welchem gesucht wird
	 * @return true, wenn ein Knoten gefunden wurde
	 */
	public boolean recursivFinde(Data d)
	{
		return rekursivFindeKnoten(kopf.getRechts(), d) != null;
	}

	private void printPreorder(BaumKnoten n)
	{
		if (n != nullKnoten)
		{
			int e = n.bestimmeEbene((Data) n.getSchluessel());

			if (e > 0)
				System.out.println("Ausnahme: " + n.toString());// _debug.fine("Ausnahme:
																												// " + n.toString());
			else
				System.out.println("Berechtigung: " + n.toString());// _debug.fine("Berechtigung:
																														// " +
																														// n.toString());
			printPreorder(n.getLinks());
			printPreorder(n.getRechts());
		}
	}

	/**
	 * Gibt die Baumstruktur in einer Map zurueck. Die Traversierungsstrategie ist
	 * hierbei Preorder. Jedes Element der Ebene 0 des Baumes wird als Schluessel
	 * der Map gespeichert, die Elemente des untergeordneten Teilbaumes werden
	 * dazu in einer Liste als Wert der Map gespeichert.
	 * 
	 * @return die Baumstruktur als Map
	 */
	public Map<Data, List<Data>> getPreorder()
	{

		Map<Data, List<Data>> map = new HashMap<Data, List<Data>>();
		BaumKnoten r = kopf.getRechts();

		while (r != nullKnoten)
		{

			Data key = null;
			List<Data> list = new ArrayList<Data>();

			if (r.getSchluessel() != null)
			{
				key = (Data) r.getSchluessel();
			}

			BaumKnoten l = r.getLinks();
			while (l != nullKnoten)
			{

				if (l.getSchluessel() != null)
				{
					list.add((Data) l.getSchluessel());
				}

				l = l.getLinks();

			}

			if (key != null)
			{
				// System.out.println("map: " + key);
				map.put(key, list);
			}

			r = r.getRechts();

		}

		return map;

	}

	private void printPostorder(BaumKnoten n)
	{
		if (n != nullKnoten)
		{
			printPostorder(n.getLinks());
			printPostorder(n.getRechts());
			_debug.fine(n.toString());
		}
	}

	private void printInorder(BaumKnoten n)
	{
		if (n != nullKnoten)
		{
			printInorder(n.getLinks());
			_debug.fine(n.toString());
			printInorder(n.getRechts());
		}
	}

	private void printLevelorder(Queue q)
	{
		while (!q.isEmpty())
		{
			BaumKnoten n = (BaumKnoten) q.leave();
			if (n.getLinks() != nullKnoten)
				q.enter(n.getLinks());
			if (n.getRechts() != nullKnoten)
				q.enter(n.getRechts());
			_debug.fine(n.toString());
		}
	}

	/**
	 * Ausgabe der Baumstruktur nach den verschieden Traversierungsstrategieen
	 * 
	 * @param order
	 */
	public void traversiere(int order)
	{
		switch (order)
		{
		case INORDER:
			printInorder(kopf.getRechts());
			break;
		case PREORDER:
			printPreorder(kopf.getRechts());
			break;
		case POSTORDER:
			printPostorder(kopf.getRechts());
			break;
		case LEVELORDER:
			Queue queue = new ArrayQueue();
			queue.enter(kopf.getRechts());
			printLevelorder(queue);
			break;
		default:
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
		StringBuffer sb = new StringBuffer();
		if (kopf.getRechts() != nullKnoten)
			kopf.getRechts().printKnoten(sb, 0, nullKnoten);
		return sb.toString();
	}

}
