/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.einstellungen;

import java.io.File;

import org.eclipse.core.runtime.Status;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.basislib.Activator;
import de.bsvrz.buv.rw.basislib.login.UrlasserDialog;
import de.bsvrz.buv.rw.rw.dav.BenutzerAngemeldet;
import de.bsvrz.buv.rw.rw.dav.BenutzerEreignis;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.IBenutzerZuhoerer;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungsAbbauWarten;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse liefert den Zugriff auf die Einstellungen fr:<br>
 * <ul>
 * <li>Benutzer Lokal</li>
 * <li>Benutzer Netzweit</li>
 * <li>Allgemein Lokal</li>
 * <li>Allgemein Netzweit</li>
 * </ul>
 * <br>
 * Der Zugriff wird als Instanz der Klasse {@link EinstellungenWaehler}
 * geliefert. Wurden Parameter verndert <b>muss</b> darauf geachtet werden, mit
 * den Methoden {@link EinstellungenWaehler#sichereSpeicherAllgemeinLokal()},
 * {@link EinstellungenWaehler#sichereSpeicherAllgemeinNetzweit()},
 * {@link EinstellungenWaehler#sichereSpeicherBenutzerLokal()} bzw.
 * {@link EinstellungenWaehler#sichereSpeicherBenutzerNetzweit()} die jeweiligen
 * {@link EinstellungenSpeicher} zu sichern.
 * 
 * @author Dambach Werke GmbH
 * 
 */
public class EinstellungenWaehler implements IDaVVerbindungsAbbauWarten
{
	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	/**
	 * Speicher fr allgemeie, lokale Einstellungen
	 */
	private EinstellungenSpeicher _speicherAllgemeinLokal = null;

	/**
	 * Speicher fr allgemeine, netzweite Einstellungen
	 */
	private EinstellungenSpeicher _speicherAllgemeinNetzweit = null;

	/**
	 * Speicher fr benutzerspezifische, lokale Einstellungen
	 */
	private EinstellungenSpeicher _speicherBenutzerLokal = null;

	/**
	 * Speicher fr benutzerspezifische, netzweite Einstllungen
	 */
	private EinstellungenSpeicher _speicherBenutzerNetzweit = null;

	/**
	 * {@link IBenutzerZuhoerer}, ber den die Information ber den aktuell
	 * angemeldeten Bentutzer bezogen wird.
	 */
	private IBenutzerZuhoerer _benutzerZuhoerer = null;

	/**
	 * Speichert den Namen des letzten Benutzers des {@link EinstellungenSpeicher}
	 * . Damit lsst sich dieser fr den Benutzer sichern, wenn sich der Benutzer
	 * abgemeldet hat.
	 */
	private String _benutzername = null;

	/**
	 * Prferenzspeicher
	 */
	private IPreferenceStore _praeferenzSpeicher = null;

	/**
	 * DaVKommunikation, die Initialisierung erfolgt im Konstruktor
	 */
	private DaVKommunikation _dav = null;

	/**
	 * Privater Konstruktor, registriert einmalig einen {@link IBenutzerZuhoerer}
	 * bei der Klasse {@link BenutzerAngemeldet}, um stets ber den angemeldeten
	 * Benutzer informiert zu sein.
	 * 
	 */
	private EinstellungenWaehler()
	{
		_dav = DaVKommunikation.getInstanz();

		_praeferenzSpeicher = PlatformUI.getPreferenceStore();
		_benutzerZuhoerer = new IBenutzerZuhoerer()
		{

			public void benutzerNeuAngemeldet(BenutzerEreignis benutzerEreignis)
			{
				System.out
						.println("EinstellungenWaehler.benutzerNeuAngemeldet() => Benutzer: "
								+ benutzerEreignis);
				// Durchfhren der Speicherung, da ja ein neuer Benutzer angemeldet
				// wird.
				sichereAktiveSpeicherVorBenutzerwechsel();
				// Neu angemeldeten Benutzer aus dem Event ziehen und in _benutername
				// ablegen.
				_benutzername = benutzerEreignis.getBenutzer();
			}

		};
		BenutzerAngemeldet.getInstanz().hinzufuegenBenutzerZuhoerer(
				_benutzerZuhoerer);
		DaVVerbindung.getInstanz().hinzufuegenVerbindungsabbauWarten(this);
	}

	public static EinstellungenWaehler getInstanz()
	{
		return Innen.INSTANZ;
	}

	private static class Innen
	{
		private static EinstellungenWaehler INSTANZ = new EinstellungenWaehler();
	}

	// /**
	// * Liefert die Singleton-Instanz der Klasse {@link EinstellungenWaehler}.
	// *
	// * @return Singleton-Instanz der Klasse {@link EinstellungenWaehler}
	// */
	// public static synchronized EinstellungenWaehler getInstanz()
	// {
	// if (_instanz == null)
	// {
	// _instanz = new EinstellungenWaehler();
	// }
	//		
	// // Logger m_log = Logger.getLogger("EinstellungenWaehler");
	// // m_log.setLevel(Level.ALL);
	// // try
	// // {
	// // FileHandler fh = new FileHandler("C:\\rw_test.log");
	// // fh.setFormatter(new SimpleFormatter());
	// // m_log.addHandler(fh);
	// // } catch (SecurityException e)
	// // {
	// // e.printStackTrace();
	// // } catch (IOException e)
	// // {
	// // e.printStackTrace();
	// // }
	// //
	// // if (m_log != null)
	// // {
	// // synchronized (m_log)
	// // {
	// // m_log.log(Level.CONFIG, "Logge.");
	// // }
	// // }
	// //
	// // m_log.log(Level.WARNING, "Instanz: " + _instanz);
	//
	//		
	// return _instanz;
	// }

	/**
	 * Liefert den einen {@link EinstellungenSpeicher} nach Wahl ber den
	 * Enumerator {@link EinstellungsArt}. In der Regel sollte mit dieser Methode
	 * ein {@link EinstellungenSpeicher} gewhlt werden, auf dem Einstellungen
	 * vorgenommen werden knnen (hinzufgen, bearbeiten, lschen und auslesen).<br>
	 * Der aktuelle Benutzer wird durch einen {@link IBenutzerZuhoerer} bestimmt. <br>
	 * <br>
	 * <b>Achtung:</b> Es ist darauf zu Achten, dass nderungen jeweils
	 * gelieferten {@link EinstellungenSpeicher} auch gesichert werden
	 * <i>mssen</i>. Die entsprechenden Methoden befinden sich weiter unten in
	 * dieser Klasse.
	 * 
	 * @param einstellungArt
	 *          Gibt an, welcher {@link EinstellungenSpeicher} geliefert werden
	 *          soll.
	 * @return Liefert den {@link EinstellungenSpeicher} oder <code>null</code>,
	 *         wenn der Einstellungenspeicher nicht bezogen werden konnte. Dies
	 *         kann bei netzweiten allgemeinen oder benutzerspezifischen
	 *         Einstellungen der Fall sein, wenn keine Verbindung zum DaV
	 *         vorhanden ist.
	 */
	public EinstellungenSpeicher getEinstellungen(EinstellungsArt einstellungArt)
	{
		EinstellungenSpeicher returnSpeicher = null;

		if (_benutzername == null)
		{
			_benutzername = BenutzerAngemeldet.getInstanz().getName();
		}

		switch (einstellungArt)
		{
		case allgemeinLokal:
			_debug.fine("Allgemein Lokal");
			initAllgemeinLokal();
			returnSpeicher = _speicherAllgemeinLokal;
			break;
		case allgemeinNetzweit:
			_debug.fine("Allgemein Netzweit");
			initAllgemeinNetzweit();
			returnSpeicher = _speicherAllgemeinNetzweit;
			break;
		case benutzerLokal:
			_debug.fine("Benutzer Lokal");
			initBenutzerLokal();
			returnSpeicher = _speicherBenutzerLokal;
			break;
		case benutzerNetzweit:
			_debug.fine("Benutzer Netzweit");
			initBenzutzerNetzweit();
			returnSpeicher = _speicherBenutzerNetzweit;
			break;
		}

		return returnSpeicher;
	}

	/**
	 * Fhrt die Initialisierung des allgemeinen, lokalen
	 * {@link EinstellungenSpeicher} durch.
	 * 
	 */
	private void initAllgemeinLokal()
	{
		if (_speicherAllgemeinLokal == null)
		{
			_speicherAllgemeinLokal = new EinstellungenSpeicher();
			// } //TODO AUS

			String pfadBasis = _praeferenzSpeicher
					.getString(EinstellungenBezeichner.psPfadEinstellungen);

			if (pfadBasis.length() > 0)
			{

				if (pfadBasis.substring(pfadBasis.length() - 1).equals(";"))
				{
					pfadBasis = pfadBasis.substring(0, pfadBasis.length() - 1);
				}

				File fileBerechnen = new File(pfadBasis);
				String pfadAbsolut = fileBerechnen.getAbsolutePath() + File.separator
						+ EinstellungenBezeichner.nameXmlDateiLokaleAllgemeineEinstellungen;

				// String pfadAbsolut = _praeferenzSpeicher
				// .getString(EinstellungenBezeichner.psPfadEinstellungen)
				// + EinstellungenBezeichner.nameXmlDateiLokaleAllgemeineEinstellungen;

				File file = new File(pfadAbsolut);

				// Logger m_log = Logger.getLogger("initAllgemeinLokal");
				// m_log.setLevel(Level.ALL);
				// try
				// {
				// FileHandler fh = new FileHandler("C:\\rw_test_einleser.log");
				// fh.setFormatter(new SimpleFormatter());
				// m_log.addHandler(fh);
				// } catch (SecurityException e)
				// {
				// e.printStackTrace();
				// } catch (IOException e)
				// {
				// e.printStackTrace();
				// }
				//
				// if (m_log != null)
				// {
				// synchronized (m_log)
				// {
				// m_log.log(Level.CONFIG, "ESP: " + _speicherAllgemeinLokal);
				// m_log.log(Level.CONFIG, "Logge. Einlesen von: " + pfadAbsolut);
				// m_log.log(Level.CONFIG, "PfadEinstellungen: " + we);
				//
				// }
				// }

				if (file.exists())
				{
					_speicherAllgemeinLokal.einlesenXmlAusDatei(pfadAbsolut);
				}
			}
		}//TODO EIN
	}

	/**
	 * Fhrt die Initialisierung des allgemeinen, netzweiten
	 * {@link EinstellungenSpeicher} durch.
	 * 
	 */
	private void initAllgemeinNetzweit()
	{
		if (_dav.istVerbunden())
		{
			if (_speicherAllgemeinNetzweit == null)
			{
				_speicherAllgemeinNetzweit = new EinstellungenSpeicher();
			}

			String xmlStruktur = _dav.getEinstellungenAllgemeinNetzweit();

			if (xmlStruktur != null)
			{
				_speicherAllgemeinNetzweit.einlesenXmlAusString(xmlStruktur);
			} else
			{
				_speicherAllgemeinNetzweit = new EinstellungenSpeicher();
			}
		}

	}

	/**
	 * Fhrt die Initialisierung des benutzerspezifischen, lokalen
	 * {@link EinstellungenSpeicher} durch.
	 * 
	 */
	private void initBenutzerLokal()
	{

		// _benutzername = BenutzerAngemeldet.getInstanz().getName().toLowerCase();
		// LOWER entfernt
		_benutzername = BenutzerAngemeldet.getInstanz().getName();

		if (_speicherBenutzerLokal == null)
		{
			_speicherBenutzerLokal = new EinstellungenSpeicher();

			// String pfadAbsolut = _praeferenzSpeicher
			// .getString(EinstellungenBezeichner.psPfadEinstellungen)
			// + "einstellungen_benutzer_" + _benutzername + ".xml";

			String pfadBasis = _praeferenzSpeicher
					.getString(EinstellungenBezeichner.psPfadEinstellungen);

			if (pfadBasis.substring(pfadBasis.length() - 1).equals(";"))
			{
				pfadBasis = pfadBasis.substring(0, pfadBasis.length() - 1);
			}

			File fileBerechnen = new File(pfadBasis);
			String pfadAbsolut = fileBerechnen.getAbsolutePath() + File.separator
					+ "einstellungen_benutzer_" + _benutzername + ".xml";

			File file = new File(pfadAbsolut);
			if (file.exists())
			{
				_speicherBenutzerLokal.einlesenXmlAusDatei(pfadAbsolut);
			}

			// Laden veranlassen
		}

		if (_benutzername != BenutzerAngemeldet.getInstanz().getName())
		{
			sichereAktiveSpeicherVorBenutzerwechsel();
			String pfadAbsolut = _praeferenzSpeicher
					.getString(EinstellungenBezeichner.psPfadEinstellungen)
					+ "einstellungen_benutzer_"
					+ BenutzerAngemeldet.getInstanz().getName() + ".xml";

			File file = new File(pfadAbsolut);
			_speicherBenutzerLokal = new EinstellungenSpeicher();
			if (file.exists())
			{
				_speicherBenutzerLokal.einlesenXmlAusDatei(pfadAbsolut);
			}
		}
	}

	/**
	 * Fhrt die Initialisierung des benutzerspezifischen, netzweiten
	 * {@link EinstellungenSpeicher} durch.
	 * 
	 */
	private void initBenzutzerNetzweit()
	{
		if (_dav.istVerbunden())
		{
			if (_speicherBenutzerNetzweit == null)
			{
				_speicherBenutzerNetzweit = new EinstellungenSpeicher();
			}

			String xmlStruktur = _dav
					.getVoreinstellungenBenutzerNetzweit(_benutzername);

			if (xmlStruktur != null)
			{
				_speicherBenutzerNetzweit.einlesenXmlAusString(xmlStruktur);
			} else
			{
				_speicherBenutzerNetzweit = new EinstellungenSpeicher();
			}
		} else
		{
			_speicherBenutzerNetzweit = null;
		}
	}

	/**
	 * Sichert die Speicher fr Benutzer lokal und Benutzer netzweit, wenn das
	 * Rahmenwerk beendet wird. Die Speicher fr allgemeine netzweite und
	 * allgemein Lokal <b>mssen</b> manuell durch den Plug-In Entwickler
	 * gesichert werden.
	 */
	public void sichereSpeicher()
	{
		_debug.fine("SICHERE SPEICHER");
		// sichereSpeicherAllgemeinLokal();
		// sichereSpeicherAllgemeinNetzweit();
		sichereSpeicherBenutzerLokal();
		sichereSpeicherBenutzerNetzweit();

		// Sichern
		// if (_speicherAllgemeinLokal != null)
		// {
		// String xmlStruktur = _speicherAllgemeinLokal.ausgabeXmlFormatiert();
		// if (pfad(_praeferenzSpeicher
		// .getString(EinstellungenBezeichner.psPfadEinstellungen)))
		// {
		// String pfadAbsolut = _praeferenzSpeicher
		// .getString(EinstellungenBezeichner.psPfadEinstellungen)
		// + EinstellungenBezeichner.nameXmlDateiLokaleAllgemeineEinstellungen;
		// System.out.println(getClass().getName()
		// + "::sichereSpeichher in Pfad: " + pfadAbsolut);
		// _speicherAllgemeinLokal.schreibeString(pfadAbsolut, xmlStruktur, true);
		// } else
		// {
		// String pfadAbsolut = "C:\\"
		// + EinstellungenBezeichner.nameXmlDateiLokaleAllgemeineEinstellungen
		// + ".sicherung";
		// System.err.println(getClass().getName()
		// + "::sichereSpeicher In den Prferenzen wurde kein Pfad zum "
		// + "sichern der lokalen, benutzerbergreifenden Einstellungen "
		// + "angegeben.\nSichere fr Auswertungszwecke in Datei "
		// + pfadAbsolut + ".");
		// _speicherAllgemeinLokal.schreibeString(pfadAbsolut, xmlStruktur, true);
		// }
		// }
		// if (_speicherAllgemeinNetzweit != null)
		// {
		// String xmlStruktur = _speicherAllgemeinNetzweit.ausgabeXmlFormatiert();
		// if (_dav.istVerbunden())
		// {
		// _dav.setVoreinstellungenAllgemeinNetzweit(xmlStruktur);
		// } else
		// {
		// String sicherungsPfad =
		// "C:\\benutzereinstellungen_netzweit_allgemein.xml";
		// _speicherAllgemeinNetzweit.schreibeString(sicherungsPfad, xmlStruktur,
		// true);
		// System.err.println(getClass().getName()
		// + "::sichereSpeicher Der Speicher fr die allgemeinen, "
		// + "netzweiten Einstellungen konnte nicht gesichtert werden, da "
		// + "keine Verbindung zum DaV vorhanden ist!\n"
		// + "Die Einstellungen wurden un der Datei " + sicherungsPfad
		// + " hinterlegt.");
		// }
		// }
		// if (_speicherBenutzerLokal != null)
		// {
		// if (pfad(_praeferenzSpeicher
		// .getString(EinstellungenBezeichner.psPfadEinstellungen)))
		// {
		// String pfadAbsolut = _praeferenzSpeicher
		// .getString(EinstellungenBezeichner.psPfadEinstellungen)
		// + "einstellungen_benutzer_" + _benutzername + ".xml";
		// System.out.println(getClass().getName()
		// + "::sichereSpeichher in Pfad: " + pfadAbsolut);
		//
		// _speicherBenutzerLokal.schreibeString(pfadAbsolut,
		// _speicherBenutzerLokal.ausgabeXmlFormatiert(), true);
		// } else
		// {
		// System.err.println(getClass().getName()
		// + "::sichereSpeicher \n\t=> Der Pfad "
		// + _praeferenzSpeicher
		// .getString(EinstellungenBezeichner.psPfadEinstellungen)
		// + " ist nicht existent und konnte nicht angelegt werden!");
		// }
		//
		// }
		// if (_speicherBenutzerNetzweit != null)
		// {
		// if (_dav.istVerbunden())
		// {
		// String xmlStruktur = _speicherBenutzerNetzweit.ausgabeXmlFormatiert();
		// _dav.setVoreinstellungenBenutzerNetzweit(xmlStruktur, _benutzername);
		// System.err.println("SICHERE ONLINE");
		// } else
		// {
		// String xmlStruktur = _speicherBenutzerNetzweit.ausgabeXmlFormatiert();
		// String sicherungsPfad = "C:\\benutzereinstellungen_netzweit_"
		// + _benutzername + ".xml";
		// _speicherBenutzerNetzweit.schreibeString(sicherungsPfad, xmlStruktur,
		// true);
		//
		// System.err.println(getClass().getName()
		// + "::sichereSpeicher Der Speicher fr die benutzerspezifischen, "
		// + "netzweiten Einstellungen konnte nicht gesichtert werden, da "
		// + "keine Verbindung zum DaV vorhanden ist!\n"
		// + "Die Einstellungen wurden un der Datei " + sicherungsPfad
		// + " hinterlegt.");
		// }
		// }
		// else
		// {
		// System.err.println("_speicherBenutzerNetzweit == NULL!!!");
		// }
	}

	/**
	 * Sichert den {@link EinstellungenSpeicher} fr allgemeine Einstellungen
	 * lokal.
	 * 
	 * @return Liefert <code>true</code>, wenn erfolgreich gespeichert werden
	 *         konnte.
	 */
	public boolean sichereSpeicherAllgemeinLokal()
	{
		boolean erfolgreich = false;

		if (_speicherAllgemeinLokal != null)
		{
			_debug.fine(getClass().getName()
					+ " => Sichere Speicher fr allgemeine Einstelungen lokal.");

			String xmlStruktur = _speicherAllgemeinLokal.ausgabeXmlFormatiert(true);

			String pfadBasis = _praeferenzSpeicher
					.getString(EinstellungenBezeichner.psPfadEinstellungen);

			if (pfadBasis.substring(pfadBasis.length() - 1).equals(";"))
			{
				pfadBasis = pfadBasis.substring(0, pfadBasis.length() - 1);
			}

			if (existiertPfad(pfadBasis))
			{
				File fileBerechnen = new File(pfadBasis);
				String pfadAbsolut = fileBerechnen.getAbsolutePath() + File.separator
						+ EinstellungenBezeichner.nameXmlDateiLokaleAllgemeineEinstellungen;

				System.out.println("PFAD: " + pfadAbsolut);

				erfolgreich = _speicherAllgemeinLokal.schreibeString(pfadAbsolut,
						xmlStruktur, true);
			} else
			{
				String pfadAbsolut = "C:\\"
						+ EinstellungenBezeichner.nameXmlDateiLokaleAllgemeineEinstellungen
						+ ".sicherung";

				String msg = getClass().getName()
						+ "::sichereSpeicher In den Prferenzen wurde kein Pfad zum "
						+ "sichern der lokalen, benutzerbergreifenden Einstellungen "
						+ "angegeben.\nSichere fr Auswertungszwecke in Datei "
						+ pfadAbsolut + ".";

				Activator.getDefault().getLog().log(
						new Status(Status.WARNING, Activator.PLUGIN_ID, msg));

				erfolgreich = _speicherAllgemeinLokal.schreibeString(pfadAbsolut,
						xmlStruktur, true);
			}
		}

		return erfolgreich;
	}

	/**
	 * Sichert den {@link EinstellungenSpeicher} fr allgemeine Einstellungen
	 * netzweit.
	 * 
	 * @return Liefert <code>true</code> zurck, wenn die Einstellungen
	 *         erfolgreich gespeichert werden konnten, sonst <code>false</code>.
	 */
	public boolean sichereSpeicherAllgemeinNetzweit()
	{
		boolean erfolgreich = false;

		if (_speicherAllgemeinNetzweit != null)
		{
			_debug.fine(getClass().getName()
					+ " => Sichere Speicher fr allgemeine Einstelungen netzweit.");

			String xmlStruktur = _speicherAllgemeinNetzweit
					.ausgabeXmlFormatiert(true);
			if (_dav.istVerbunden())
			{
				erfolgreich = _dav.setVoreinstellungenAllgemeinNetzweit(xmlStruktur);
			} else
			{
				String sicherungsPfad = "C:\\benutzereinstellungen_netzweit_allgemein.xml";
				_speicherAllgemeinNetzweit.schreibeString(sicherungsPfad, xmlStruktur,
						true);

				String msg = getClass().getName()
						+ "::sichereSpeicher Der Speicher fr die allgemeinen, "
						+ "netzweiten Einstellungen konnte nicht gesichtert werden, da "
						+ "keine Verbindung zum DaV vorhanden ist!\n"
						+ "Die Einstellungen wurden in der Datei " + sicherungsPfad
						+ " hinterlegt.";

				Activator.getDefault().getLog().log(
						new Status(Status.WARNING, Activator.PLUGIN_ID, msg));
			}
		}

		return erfolgreich;
	}

	/**
	 * Sichert den {@link EinstellungenSpeicher} lokal fr den aktuell
	 * angemeldeten BenutzerBenutzer. Dieser wird ber einen
	 * {@link IBenutzerZuhoerer} bestimmt.
	 */
	public void sichereSpeicherBenutzerLokal()
	{
		if (_speicherBenutzerLokal != null)
		{
			if (existiertPfad(_praeferenzSpeicher
					.getString(EinstellungenBezeichner.psPfadEinstellungen)))
			{
				String pfadBasis = _praeferenzSpeicher
						.getString(EinstellungenBezeichner.psPfadEinstellungen);

				if (pfadBasis.substring(pfadBasis.length() - 1).equals(";"))
				{
					pfadBasis = pfadBasis.substring(0, pfadBasis.length() - 1);
				}

				File fileBerechnen = new File(pfadBasis);
				String pfadAbsolut = fileBerechnen.getAbsolutePath() + File.separator
						+ "einstellungen_benutzer_" + _benutzername + ".xml";

				_speicherBenutzerLokal.schreibeString(pfadAbsolut,
						_speicherBenutzerLokal.ausgabeXmlFormatiert(true), true);
			} else
			{

				String msg = getClass().getName()
						+ "::sichereSpeicher \n\t=> Der Pfad "
						+ _praeferenzSpeicher
								.getString(EinstellungenBezeichner.psPfadEinstellungen)
						+ " zum Speichern der lokalen Einstellungen ist nicht "
						+ "existent und konnte nicht angelegt werden!";

				Activator.getDefault().getLog().log(
						new Status(Status.WARNING, Activator.PLUGIN_ID, msg));
			}

		}
	}

	/**
	 * Sichert den {@link EinstellungenSpeicher} netzweit fr den Benutzer. Dieser
	 * wird ber einen {@link IBenutzerZuhoerer} bestimmt.
	 * 
	 * @return Liefert <code>true</code>, wenn die Einstellungen fr den Benutzer
	 *         gesichert werden konnten, sonnst <code>false</code>.
	 */
	public boolean sichereSpeicherBenutzerNetzweit()
	{
		boolean re = false;

		if (_speicherBenutzerNetzweit != null)
		{
			System.out
					.println(getClass().getName()
							+ "::sichereSpeicherBenutzerNetzweit\n\t=> Sichere Speicher fr Benutzereinstelungen netzweit.");

			String xmlStruktur = _speicherBenutzerNetzweit.ausgabeXmlFormatiert(true);

			if (_dav.istVerbunden())
			{
				re = _dav.setVoreinstellungenBenutzerNetzweit(xmlStruktur,
						_benutzername);
			} else
			{
				System.out
						.println(getClass().getName()
								+ "::sichereSpeicherBenutzerNetzweit\n\t=> KEine Verbindung zum DaV vorhanden.");
			}
		}

		return re;
	}

	/**
	 * Diese Methode liefert die Einstellungen fr einen bestimmten Benutzer in
	 * Form eines {@link EinstellungenSpeicher} zurck.<br>
	 * <b>Achtung:</b> Diese Funktion sollten nur fr administrative Aufgaben
	 * verwendet werden, wenn der betreffende Benutzer nicht angemeldet ist.
	 * I.d.R. ist die Methode
	 * {@link EinstellungenWaehler#getEinstellungen(de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenWaehler.EinstellungsArt)}
	 * zu bevorzugen.
	 * 
	 * @param benutzerName
	 *          Name des Benutzers
	 * @return Liefert den {@link EinstellungenSpeicher} fr den Benutzer zurck.
	 *         Existieren fr diesen Benutzer keine Einstellungen auf dem DaV,
	 *         wird ein leerer {@link EinstellungenSpeicher} zurckgeliefert.
	 *         Existiert keine Verbindung zum DaV, wird <code>null</code>
	 *         zurckgeliefert.
	 */
	public EinstellungenSpeicher getEinstellungenSpeicherBenzutzerBestimmtNetzweit(
			String benutzerName)
	{
		EinstellungenSpeicher re = null;

		if ((_dav != null) && (_dav.istVerbunden()))
		{
			String xmlStruktur = _dav
					.getVoreinstellungenBenutzerNetzweit(benutzerName);

			// Erzeugen eines leeren Einstellungenspeichers
			EinstellungenSpeicher esp = new EinstellungenSpeicher();

			if ((xmlStruktur != null) && (xmlStruktur.length() > 0))
			{
				esp.einlesenXmlAusString(xmlStruktur);
			}

			re = esp;
		}

		return re;
	}

	/**
	 * Liefert den EinstellungenSpeicher fr einen bestimmten, lokalen Benutzer.
	 * Existiert der Benutzer oder dessen {@link EinstellungenSpeicher} noch
	 * nicht, so wird ein leerer {@link EinstellungenSpeicher} zurckgeliefert.<br>
	 * 
	 * <b>Achtung:</b> Diese Funktion sollten nur fr administrative Aufgaben
	 * verwendet werden, wenn der betreffende Benutzer nicht angemeldet ist.
	 * I.d.R. ist die Methode
	 * {@link EinstellungenWaehler#getEinstellungen(de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenWaehler.EinstellungsArt)}
	 * zu bevorzugen.
	 * 
	 * @param benutzerName
	 *          Name des Benutzers, dessen {@link EinstellungenSpeicher} geliefert
	 *          werden soll.
	 * @return Den {@link EinstellungenSpeicher} fr den Benutzer. Dieser kann
	 *         leer sein, wenn noch kein {@link EinstellungenSpeicher} fr den
	 *         Benutzer existent war.
	 */
	public EinstellungenSpeicher getEinstellungenSpeicherBenutzerBestimmtLokal(
			String benutzerName)
	{
		_benutzername = benutzerName;

		EinstellungenSpeicher speicherBenutzerLokal = new EinstellungenSpeicher();

		String pfadBasis = _praeferenzSpeicher
				.getString(EinstellungenBezeichner.psPfadEinstellungen);

		if (pfadBasis.substring(pfadBasis.length() - 1).equals(";"))
		{
			pfadBasis = pfadBasis.substring(0, pfadBasis.length() - 1);
		}

		File fileBerechnen = new File(pfadBasis);
		String pfadAbsolut = fileBerechnen.getAbsolutePath() + File.separator
				+ "einstellungen_benutzer_" + _benutzername + ".xml";

		File file = new File(pfadAbsolut);
		if (file.exists())
		{
			speicherBenutzerLokal.einlesenXmlAusDatei(pfadAbsolut);
		}

		return speicherBenutzerLokal;
	}

	/**
	 * Sichert den {@link EinstellungenSpeicher} netzweit fr einen bestimmten
	 * Benutzer.<br>
	 * <b>Achtung:</b> Diese Funktion sollten nur fr administrative Aufgaben
	 * verwendet werden, wenn der betreffende Benutzer nicht angemeldet ist.
	 * I.d.R. sind die Methoden {@link EinstellungenWaehler#sichereSpeicher()}
	 * bzw. {@link EinstellungenWaehler#sichereSpeicherBenutzerNetzweit()} zu
	 * bevorzugen.
	 * 
	 * @param benutzerName
	 *          Name des Benutzers, fr den der {@link EinstellungenSpeicher}
	 *          gesichert werden soll.<br>
	 *          <b><u>Achtung:</u></b> Dieser Benutzer muss existieren, da sonst
	 *          die Einstellungen nicht gespeichert werden knnen!
	 * @param speicher
	 *          Der {@link EinstellungenSpeicher} des Benutzers, der gesichert
	 *          werden soll. Darf nicht <code>null</code> sein.
	 * @return Liefert <code>true</code>, wenn der {@link EinstellungenSpeicher}
	 *         eines Benutzers erfolgreich gesichert werden konnte. Liefert
	 *         <code>false</code>, wenn keine Verbindung zum DaV existiert oder
	 *         der angegebene Benutzer nicht existiert.
	 */
	public boolean sichereEinstellungenSpeicherBenutzerBestimmtNetzweit(
			String benutzerName, EinstellungenSpeicher speicher)
	{
		boolean re = false;

		if ((benutzerName != null) && (benutzerName.length() > 0))
		{
			if (speicher != null)
			{
				if (_dav != null)
				{
					String xmlStruktur = speicher.ausgabeXmlFormatiert(true);
					boolean erfolgreich = _dav.setVoreinstellungenBenutzerNetzweit(
							xmlStruktur, benutzerName);
					re = erfolgreich;
				}
			}
		}

		return re;
	}

	/**
	 * Sichert den {@link EinstellungenSpeicher} fr einen bestimmten Benutzer
	 * LOKAL.<br>
	 * <b>Achtung:</b> Diese Funktion sollten nur fr administrative Aufgaben
	 * verwendet werden, wenn der betreffende Benutzer nicht angemeldet ist.
	 * I.d.R. sind die Methoden {@link EinstellungenWaehler#sichereSpeicher()}
	 * bzw. {@link EinstellungenWaehler#sichereSpeicherBenutzerLokal()} zu
	 * bevorzugen.
	 * 
	 * @param benutzerName
	 *          Name des Benutzers, fr den der {@link EinstellungenSpeicher}
	 *          gesichert werden soll.
	 * @param speicher
	 *          Zu sichernder {@link EinstellungenSpeicher}.
	 * @return Liefert <code>true</code>, wenn der {@link EinstellungenSpeicher}
	 *         erfolgreich gesichert werden konnte, sonst <code>false</code>.
	 */
	public boolean sichereEinstellungenSpeicherBenutzerBestimmtLokal(
			String benutzerName, EinstellungenSpeicher speicher)
	{
		boolean erfolgreich = false;

		if (speicher != null)
		{
			if (existiertPfad(_praeferenzSpeicher
					.getString(EinstellungenBezeichner.psPfadEinstellungen)))
			{
				// String pfadAbsolut = _praeferenzSpeicher
				// .getString(EinstellungenBezeichner.psPfadEinstellungen)
				// + "einstellungen_benutzer_" + benutzerName + ".xml";
				// System.out
				// .println(getClass().getName()
				// + "::sichereEinstellungenSpeicherBenutzerBestimmtLokal Sichere
				// Speicher in Pfad: "
				// + pfadAbsolut);

				String pfadBasis = _praeferenzSpeicher
						.getString(EinstellungenBezeichner.psPfadEinstellungen);

				if (pfadBasis.substring(pfadBasis.length() - 1).equals(";"))
				{
					pfadBasis = pfadBasis.substring(0, pfadBasis.length() - 1);
				}

				File fileBerechnen = new File(pfadBasis);
				String pfadAbsolut = fileBerechnen.getAbsolutePath() + File.separator
						+ "einstellungen_benutzer_" + _benutzername + ".xml";

				try
				{
					erfolgreich = speicher.schreibeString(pfadAbsolut, speicher
							.ausgabeXmlFormatiert(true), true);

				} catch (IllegalArgumentException e)
				{
					System.out
							.println(getClass().getName()
									+ "::sichereEinstellungenSpeicherBenutzerBestimmtLokal Fehler beim Speichern\n\t=> "
									+ e.toString());
				}
			} else
			{
				_debug
						.error(getClass().getName()
								+ "::sichereEinstellungenSpeicherBenutzerBestimmtLokal Der EinstellungenSpeicher konnte nicht gesichert werden!");
			}
		}

		return erfolgreich;
	}

	/**
	 * Prft ob ein Pfad existiert. Wenn nein, wird dieser neu angelegt.
	 * 
	 * @param pfad
	 *          Pfad, auf dessen Existenz geprft werden soll.
	 */
	private boolean existiertPfad(String pfad)
	{

		String substring = pfad.substring(pfad.length() - 1);

		if (substring.equals(File.pathSeparator))
		{
			pfad = pfad.substring(0, pfad.length() - 1);
		}

		File dir = new File(pfad);
		if (dir.isDirectory())
		{
			_debug.fine(getClass().getName() + "::pfad=> Verzeichnis " + pfad
					+ " existiert.");

			return true;
		} else
		{
			_debug.fine(getClass().getName() + "::pfad=> Verzeichnis " + pfad
					+ " existiert noch nicht und wird angelegt.");

			dir.mkdir();

			_debug.fine(getClass().getName() + "::pfad=> Das Verzeichnis " + pfad
					+ " wurde erzeugt.");

			File dir_2ndcheck = new File(pfad);
			if (dir_2ndcheck.isDirectory())
			{
				_debug.fine(getClass().getName()
						+ "::pfad=> 2.ter Verzeichnisexistenzcheck: ok.");

				return true;
			} else
			{
				_debug
						.error(getClass().getName()
								+ "::pfad=> 2.ter Verzeichnisexistenzcheck: Verzeichnis konnte nicht angelegt werden.");
				return false;
			}
		}
	}

	/**
	 * Sichert die aktiven {@link EinstellungenSpeicher}, die <code>!= null</code>
	 * sind, bevor ein Benutzerwechsel statt findet.
	 * 
	 */
	private void sichereAktiveSpeicherVorBenutzerwechsel()
	{
		System.out
				.println("EinstellungenWaehler.sichereAktiveSpeicherBenutzerwechsel() => SICHERN");

		sichereSpeicher();

		// Speicher auf NULL setzen
		_speicherAllgemeinLokal = null;
		_speicherAllgemeinNetzweit = null;
		_speicherBenutzerLokal = null;
		_speicherBenutzerNetzweit = null;
	}

	/**
	 * Liefert die Art der Einstellung
	 * 
	 * @author HofmannJ
	 * 
	 */
	public static enum EinstellungsArt
	{
		/**
		 * benutzerspezifische, lokale Einstellungen
		 */
		benutzerLokal,
		/**
		 * benutzerspezifische, netzweite Einstellungen
		 */
		benutzerNetzweit,
		/**
		 * allgemeine, lokale Einstellungen
		 */
		allgemeinLokal,
		/**
		 * allgemeine, netzweite Einstellungen
		 */
		allgemeinNetzweit
	}

	public boolean DaVVerbindungsAbbauOk()
	{
		boolean returnWert = true;

		if (_speicherAllgemeinNetzweit != null)
		{
			if (_speicherAllgemeinNetzweit.istSchmutzig())
			{
				returnWert = false;
			}
		}

		if (_speicherBenutzerNetzweit != null)
		{
			if (_speicherBenutzerNetzweit.istSchmutzig())
			{
				returnWert = false;
			}
		}

		// if (_speicherBenutzerLokal != null)
		// {
		// if (_speicherBenutzerLokal.istSchmutzig())
		// {
		// returnWert = false;
		// }
		// }

		if (returnWert)
		{
			_debug.fine("EinstellungenWaehler.DaVVerbindungsAbbauOk() => TRUE");
		} else
		{
			_debug.fine("EinstellungenWaehler.DaVVerbindungsAbbauOk() => FALSE");
		}

		return returnWert;

	}

	/**
	 * Legt fest, ob mit oder ohne {@link UrlasserDialog} gesendet werden soll.<br>
	 * <b>WICHTIG:</b> Dieses Flag <b><i>muss</i></b> nach dem Speichern wieder
	 * zurckgesetzt werden!!!
	 * 
	 * @param sendenOhneUrlasserDialog
	 *          Wenn <code>true</code>, dann wird mit {@link UrlasserDialog}
	 *          gesendet, sonst nicht.
	 */
	public void setSendenOhneUrlasserDialog(boolean sendenOhneUrlasserDialog)
	{
		if (_dav != null)
		{
			_dav.setSendenOhneUrlasserDialog(sendenOhneUrlasserDialog);
		}

	}

	/**
	 * Prft, ob mit oder ohne {@link UrlasserDialog} gesendet wird.
	 * 
	 * @return Wenn <code>true</code>, dann wird mit {@link UrlasserDialog}
	 *         gesendet, sonst nicht.
	 */
	public boolean isSendenOhneUrlasserDialog()
	{
		if (_dav != null)
		{
			return _dav.isSendenOhneUrlasserDialog();
		} else
		{
			return false;
		}
	}
}
