package de.bsvrz.buv.rw.basislib.menu.generatoren;

import java.util.Iterator;
import java.util.concurrent.CopyOnWriteArrayList;

/**
 * Diese Klasse kann Listener darber informieren, dass das Laden bzw.
 * Aktualisieren von Men-/Symbolleiste beendet ist.
 * 
 * @author HofmannJ
 * 
 */
public class MenueLadeInformation
{
	/**
	 * Speichert die Singleton-Instanz dieser Klasse.
	 */
	private static MenueLadeInformation _instance = null;

	/**
	 * Liste der {@link IMenueLadeListener}, die ber eine Aktualisierung von
	 * Men-/Symbolleiste informiert werden sollen.
	 */
	private CopyOnWriteArrayList<IMenueLadeListener> _lListener = new CopyOnWriteArrayList<IMenueLadeListener>();

	/**
	 * Privater Konstruktor dieser Singleton-Klasse.
	 */
	private MenueLadeInformation()
	{
	}

	/**
	 * Liefert die Singleton-Instanz dieser Klasse.
	 * 
	 * @return Singleton-Instanz dieser Klasse
	 */
	public static MenueLadeInformation getInstance()
	{
		if (_instance == null)
		{
			synchronized (MenueLadeInformation.class)
			{
				if (_instance == null)
				{
					_instance = new MenueLadeInformation();
				}
			}
		}

		return _instance;
	}

	/**
	 * Fgt einen {@link IMenueLadeListener} hinzu der ab dem Zeitpunkt des
	 * Hinzufgens darber informiert wird, wenn die Men-/Symbolleiste
	 * aktualisiert wurde.
	 * 
	 * @param listener
	 *          Hinzuzufgender {@link IMenueLadeListener}.
	 */
	public void addListener(IMenueLadeListener listener)
	{
		if ((listener != null) && (!_lListener.contains(listener)))
		{
			_lListener.add(listener);
		}
	}

	/**
	 * Entfernt einen {@link IMenueLadeListener}, der ab sofort nicht mehr ber
	 * eine Aktualisierung der Men-/Symbolleiste informiert wird.
	 * 
	 * @param listener
	 *          Zu entfernender {@link IMenueLadeListener}.
	 * @return Liefert <code>true</code> wenn der {@link IMenueLadeListener}
	 *         entfernt werden konnte, sonst <code>false</code>.
	 */
	public boolean removeListener(IMenueLadeListener listener)
	{
		boolean listenerErfolgreichEntfernet = false;

		if ((listener != null) && (_lListener.contains(listener)))
		{
			listenerErfolgreichEntfernet = _lListener.remove(listener);
		}

		return listenerErfolgreichEntfernet;
	}

	/**
	 * Benachrichtigt alle angemeldeten {@link IMenueLadeListener} darber, dass
	 * eine Aktualisierung der Menleiste statt gefunden hat.
	 */
	protected void benachrichtigeListenerUeberAktualisierungMenueleiste()
	{
		Iterator<IMenueLadeListener> it = _lListener.iterator();
		while (it.hasNext())
		{
			IMenueLadeListener menueLadeListener = it.next();
			if (menueLadeListener != null)
			{
				menueLadeListener.menueLeisteGeladen();
			}
		}
	}

	/**
	 * Benachrichtigt alle angemeldeten {@link IMenueLadeListener} darber, dass
	 * eine Aktualisierung der Symbolleiste statt gefunden hat.
	 */
	protected void benachrichtigeListenerUeberAktualisierungSymbolleiste()
	{
		Iterator<IMenueLadeListener> it = _lListener.iterator();
		while (it.hasNext())
		{
			IMenueLadeListener menueLadeListener = it.next();
			if (menueLadeListener != null)
			{
				menueLadeListener.symbolLeisteGeladen();
			}
		}
	}
}
