/*
 * Copyright 2007 by Kappich Systemberatung, Aachen
 * Copyright 2004 by Kappich+Kni Systemberatung Aachen (K2S)
 * 
 * This file is part of de.bsvrz.pat.sysbed.
 * 
 * de.bsvrz.pat.sysbed is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.pat.sysbed is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.pat.sysbed; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package de.bsvrz.pat.sysbed.dataview;

import de.bsvrz.dav.daf.main.DataState;
import de.bsvrz.sys.funclib.debug.Debug;

import javax.swing.*;
import javax.swing.border.LineBorder;
import java.awt.*;
import java.util.*;
import java.util.List;

/**
 * Diese Klasse bietet eine hierarchische Struktur fr einen Datensatz aus dem Datenverteiler. Es werden Attribute, Listen, Arrays von Attributen und Arrays von
 * Listen bercksichtigt.
 *
 * @author Kappich Systemberatung
 * @version $Revision: 5052 $
 * @see Dataset
 */
public class RowData implements RowListener {

	/** Der Debug-Logger */
	private final Debug _debug = Debug.getLogger();

	/** speichert den anzuzeigenden Text dieses Feldes */
	private String _value = "";

	/** Gibt an, ob sich dieses Objekt um ein Array von Listen/Attributen handelt, oder nicht. */
	private boolean _isArray = false;

	/** speichert die Komponente, die aus sich selbst und allen Nachfolgern besteht */
	private JComponent _component;

	/** speichert die initiale Breite dieser Komponente */
	private int _initialWidth = 0;

	/** speichert die optimale Spaltenbreite */
	private int _optimalColumnWidth = 0;

	/**
	 * Speichert alle Nachfolger in einer Liste. Die Nachfolger sind vom Typ {@link RowSuccessor} oder {@link RowData}, je nachdem, ob _isArray true oder false
	 * ist.
	 */
	private final List<Object> _successors = new ArrayList<Object>();

	/** Speichert einen empfangenen Datensatz */
	private final DataTableObject _dataTableObject;

	private final HeaderGrid _headerGrid;


	/* ############## Konstruktor ##################### */
	/** leerer Standardkonstruktor */
	public RowData() {
		this(null, null);
	}

	/**
	 * Stellt die Daten eines Ergebnisdatensatzes in der Onlinetabelle dar.
	 *
	 * @param headerGrid      Hierarchische Struktur der Darstellung
	 * @param dataTableObject ein Datensatz der Online-Tabelle
	 */
	public RowData(final HeaderGrid headerGrid, final DataTableObject dataTableObject) {
		_headerGrid = headerGrid;
		_dataTableObject = dataTableObject;
	}

	/**
	 * Gibt die Komponente zurck, die sich selbst und alle ihre Nachfolger darstellt.
	 *
	 * @return Komponente, die sich selbst und alle ihre Nachfolger darstellt
	 */
	public JComponent getComponent() {
		if(_component == null) {
			createComponent();
		}
		return _component;
	}

	/**
	 * Setzt die initiale Breite der Komponente, die sich selbst und alle Nachfolger darstellt.
	 *
	 * @param width die neue Breite dieser Komponente
	 */
	public void setInitialWidth(int width) {
		_initialWidth = width;
	}

	/**
	 * Gibt die initiale Spaltenbreite zurck.
	 *
	 * @return initiale Spaltenbreite
	 */
	public int getInitialWidth() {
		return _initialWidth;
	}


	/**
	 * Erzeugt die Komponente, die sich selbst und alle Nachfolger darstellt.
	 *
	 * @return die Komponente, die sich selbst und alle Nachfolger darstellt
	 */
	public JComponent createComponent() {
		if(_dataTableObject == null) {    // das Objekt wird hier nicht bentigt, darum wurde es auch nicht zugewiesen
			// Attribut -> keine Nachfolger
			if(_successors.isEmpty()) {
				_component = new RowElement(_value);
			}
			else {
				if(!_isArray) {    // handelt sich um eine Liste - Eintrge sind RowData
					JPanel panel = new JPanel(new BorderLayout());
					GridBagLayout gbl = new GridBagLayout();
					GridBagConstraints gbc;
					JPanel listPanel = new JPanel();
					listPanel.setLayout(gbl);
					int column = 0;

					for(Iterator iterator = _successors.iterator(); iterator.hasNext();) {
						Object object = iterator.next();
						if(object instanceof RowData) {
							RowData rowData = (RowData)object;
							JComponent row = rowData.getComponent();
							gbc = makeGBC(column++, 0, 1, 1, 100, 100);
							gbc.fill = GridBagConstraints.BOTH;
							gbl.setConstraints(row, gbc);
							listPanel.add(row);
						}
						else {
							_debug.error("Daten mssen vom Typ RowData sein!");
						}
					}
					panel.add(listPanel, BorderLayout.CENTER);
					_component = panel;
				}
				else {        // es handelt sich um ein Array (von Lists, von Attributen)
					JPanel panel = new JPanel(new BorderLayout());
					GridBagLayout gbl = new GridBagLayout();
					GridBagConstraints gbc;
					JPanel arrayPanel = new JPanel();
					arrayPanel.setLayout(gbl);
					int row = 0;

					for(Iterator iterator = _successors.iterator(); iterator.hasNext();) {
						Object object = iterator.next();
						if(object instanceof RowSuccessor) {
							RowSuccessor succ = (RowSuccessor)object;
							GridBagLayout gbl2 = new GridBagLayout();
							GridBagConstraints gbc2;
							JPanel successorPanel = new JPanel();
							successorPanel.setLayout(gbl2);
							int column = 0;

							List succs = succ.getSuccessors();
							for(Iterator iterator1 = succs.iterator(); iterator1.hasNext();) {
								RowData rowData = (RowData)iterator1.next();
								JComponent rowComp = rowData.getComponent();
								gbc2 = makeGBC(column++, 0, 1, 1, 100, 100);
								gbc2.fill = GridBagConstraints.BOTH;
								gbl2.setConstraints(rowComp, gbc2);
								successorPanel.add(rowComp);
							}
							JPanel succPanel = new JPanel();
							succPanel.setLayout(new BorderLayout());
							succPanel.add(successorPanel, BorderLayout.CENTER);

							gbc = makeGBC(0, row++, 1, 1, 100, 100);
							gbc.fill = GridBagConstraints.BOTH;
							gbl.setConstraints(succPanel, gbc);
							arrayPanel.add(succPanel);
						}
						else {
							_debug.error("Daten mssen vom Typ RowSuccessor sein!");
						}
					}
					panel.add(arrayPanel, BorderLayout.CENTER);
					_component = panel;
				}
			}
		}
		else {
			if(_dataTableObject.getData() == null) {
				// in ResultData sind keine Daten vorhanden -> anzeigen des DataState!
				// Ausgabetext zusammenstellen und Hintergrundfarbe des Textes ermitteln
				String outputText = "";
				Color color;
				DataState resultDataState = _dataTableObject.getDataState();
				if(resultDataState == DataState.NO_DATA) {
					outputText = "keine Daten";
					color = Color.green;
				}
				else if(resultDataState == DataState.NO_SOURCE) {
					outputText = "keine Daten (keine Quelle)";
					color = Color.orange;
				}
				else if(resultDataState == DataState.NO_RIGHTS) {
					outputText = "keine Daten (keine Rechte)";
					color = Color.red;
				}
				else if(resultDataState == DataState.POSSIBLE_GAP) {
					outputText = "keine Daten (potentielle Datenlcke)";
					color = Color.magenta;
				}
				else if(resultDataState == DataState.END_OF_ARCHIVE) {
					outputText = "Ende des Archivanfragezeitraums";
					color = Color.cyan;
				}
				else if(resultDataState == DataState.DELETED_BLOCK) {
					outputText = "Gelschter Bereich";
					color = Color.red;
				}
				else if(resultDataState == DataState.UNAVAILABLE_BLOCK) {
					outputText = "Ausgelagerter Bereich";
					color = Color.yellow;
				}
				else if(resultDataState == DataState.INVALID_SUBSCRIPTION) {
					outputText = "keine Daten (fehlerhafte Anmeldung)";
					color = Color.red;
				}
				else {
					outputText = "keine Daten (Undefinierte Objektkodierung)";
					color = Color.red;
				}

				GridBagLayout gbl = new GridBagLayout();
				GridBagConstraints gbc;

				JPanel panel = new JPanel();
				panel.setLayout(new BorderLayout());
				panel.setBorder(new LineBorder(new Color(166, 166, 166)));
				panel.setMinimumSize(new Dimension(0, 0));
				panel.setForeground(color);
				panel.setBackground(color);

				JLabel label = new JLabel(outputText);
				label.setBackground(color);
				gbc = makeGBC(0, 0, 1, 1, 100, 100);
				gbc.fill = GridBagConstraints.HORIZONTAL;
				gbl.setConstraints(label, gbc);

				panel.add(label, BorderLayout.CENTER);
				_component = panel;
			}
		}
		_optimalColumnWidth = _component.getPreferredSize().width;

		if(_dataTableObject == null) {
			if(_initialWidth > 0) {
				_component.setPreferredSize(new Dimension(_initialWidth, _component.getPreferredSize().height));
			}
		}
		else {
			if(_initialWidth > 0) {
				_component.setPreferredSize(new Dimension(_initialWidth, _component.getPreferredSize().height));
			}
		}
		return _component;
	}

	/**
	 * Hilfsfunktion zur Konstruktion des Panels. Hierber werden die Bedingungen fr die Anordnung der Elemente gesetzt.
	 *
	 * @param gridx      Spaltennummer
	 * @param gridy      Zeilennummer
	 * @param gridwidth  Anzahl der Spalten ber die das Element reicht
	 * @param gridheight Anzahl der Zeilen ber die das Element reicht
	 * @param weightx    Verteilung von zur Verfgung stehendem Platz (horizontal)
	 * @param weighty    Verteilung von zur Verfgung stehendem Platz (vertikal)
	 *
	 * @return die Bedingungen fr die Anordnung des Elements
	 */
	private GridBagConstraints makeGBC(int gridx, int gridy, int gridwidth, int gridheight, double weightx, double weighty) {
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx = gridx;
		gbc.gridy = gridy;
		gbc.gridwidth = gridwidth;
		gbc.gridheight = gridheight;
		gbc.weightx = weightx;
		gbc.weighty = weighty;
		return gbc;
	}

	/**
	 * Setzt das Flag, ob es sich hierbei um ein Array von Listen/Attributen handelt, oder nicht.
	 *
	 * @param bool true, falls es ein Array von Listen/Attributen ist, sonst false
	 */
	public void setIsArray(boolean bool) {
		_isArray = bool;
	}

	/**
	 * Gibt zurck, ob es sich um ein Array von Listen/Attributen handelt, oder nicht.
	 *
	 * @return true, falls es sich um ein Array von Listen/Attributen handelt, sonst false
	 */
	public boolean isArray() {
		return _isArray;
	}

	/**
	 * Setzt den anzuzeigenden Text.
	 *
	 * @param value darzustellenden Text
	 */
	public void setValue(String value) {
		_value = value;
	}

	/**
	 * Gibt den anzuzeigenden Text zurck.
	 *
	 * @return anzuzeigender Text
	 */
	public String getValue() {
		return _value;
	}

	/**
	 * Fgt einen Nachfolger vom Typ <code>RowSuccessor</code> oder <code>RowData</code> hinzu.
	 *
	 * @param object Nachfolger vom Typ <code>RowSuccessor</code> oder <code>RowData</code>
	 *
	 * @see RowSuccessor
	 * @see RowData
	 */
	public void addArrayElement(Object object) {
		_successors.add(object);
	}

	/**
	 * Gibt alle Nachfolger zurck. Sie knnen vom Typ RowSuccessor oder vom Typ RowData sein, je nachdem, ob es sich um ein Array von Listen/Attributen handelt,
	 * oder nicht.
	 *
	 * @return alle Nachfolger
	 */
	public List getArray() {
		return _successors;
	}


	/* ############# implementiert die Methoden des RowListener-Interfaces ############## */
	/**
	 * Gibt die fr diese Komponente optimale Spaltenbreite zurck.
	 *
	 * @return die optimale Spaltenbreite
	 */
	public int getOptimalColumnWidth() {
		return _optimalColumnWidth;
	}

	/**
	 * Setzt die Breite der Komponente, die diese Daten reprsentiert.
	 *
	 * @param width neue Breite der Komponente
	 */
	public void setWidth(int width) {
		if(_dataTableObject != null) {
			if(_dataTableObject.getData() == null) {
				_component.setPreferredSize(new Dimension(0, _component.getPreferredSize().height));
				_component.setMinimumSize(new Dimension(0, _component.getPreferredSize().height));
				_component.validate();
			}
		}
		else {
			Dimension size = new Dimension(width, _component.getSize().height);
			_component.setPreferredSize(size);
			_component.setSize(size);
			_component.setMinimumSize(size);
			_component.setMaximumSize(size);
			_component.validate();
		}
	}

	public String toString() {
		return "RowData{" + "_value='" + _value + "'" + ", _isArray=" + _isArray + ", _component=" + _component + ", _initialWidth=" + _initialWidth
		       + ", _optimalColumnWidth=" + _optimalColumnWidth + ", _successors=" + _successors + ", _dataTableObject=" + _dataTableObject + "}";
	}
}
