/*
 * Copyright 2009 by Kappich Systemberatung, Aachen
 * Copyright 2008 by Kappich Systemberatung Aachen
 * 
 * This file is part of de.bsvrz.puk.config.
 * 
 * de.bsvrz.puk.config is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.puk.config is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.puk.config; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package de.bsvrz.puk.config.main.communication.query;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.ConfigurationAuthority;
import de.bsvrz.dav.daf.main.config.MutableCollection;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.puk.config.configFile.datamodel.ConfigMutableSet;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDynamicObjectType;
import de.bsvrz.sys.funclib.debug.Debug;

import java.util.*;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;

/**
 * TBD RS dokumentieren.
 *
 * @author Kappich Systemberatung
 * @version $Revision: 6393 $
 */
public class ForeignObjectManager {

	/** DebugLogger fr Debug-Ausgaben */
	private static final Debug _debug = Debug.getLogger();

	private final ClientDavInterface _connection;

	private final DataModel _configuration;

	private final SystemObject _localAuthority;

	private final Map<String, ForeignConfigRequester> _authorityPid2Requester = new HashMap<String, ForeignConfigRequester>();

	private ScheduledExecutorService _executor;

	private HashMap<Long, SystemObject> _foreignObjects = new HashMap<Long, SystemObject>();

	private ForeignConfigReceiveCommunicator _foreignConfigReceiveCommunicator;


	public ForeignObjectManager(final ClientDavInterface connection, final DataModel configuration, final SystemObject localAuthority) {
		_connection = connection;
		_configuration = configuration;
		_localAuthority = localAuthority;
		Aspect responseAspect = configuration.getAspect("asp.antwort");
		AttributeGroup responseAtg = configuration.getAttributeGroup("atg.konfigurationsAnfrageSchnittstelleLesend");
		final DataDescription responseDescription = new DataDescription(responseAtg, responseAspect);
		_foreignConfigReceiveCommunicator = new ForeignConfigReceiveCommunicator(connection, _localAuthority, responseDescription);
	}

	public ForeignConfigRequester getForeignConfigRequester(final MutableCollection mutableCollection) {
		if(mutableCollection instanceof MutableSet) {
			MutableSet mutableSet = (MutableSet)mutableCollection;

			String managementPid = getElementsManagementPid(mutableSet);
			if(managementPid.equals(_localAuthority.getPid())) {
				return null;
			}

			synchronized(_authorityPid2Requester) {
				if(_authorityPid2Requester.containsKey(managementPid)) {
					return _authorityPid2Requester.get(managementPid);
				}
			}
			final SystemObject managementObject = _configuration.getObject(managementPid);
			synchronized(_authorityPid2Requester) {
				if(_authorityPid2Requester.containsKey(managementPid)) {
					return _authorityPid2Requester.get(managementPid);
				}
				ForeignConfigRequester foreignConfigRequester = null;
				if(managementObject instanceof ConfigurationAuthority) {
					ConfigurationAuthority foreignAuthority = (ConfigurationAuthority)managementObject;
					foreignConfigRequester = new ForeignConfigRequester(this, foreignAuthority, _foreignConfigReceiveCommunicator);
				}
				else {
					_debug.warning("Die dynamische Menge soll von " + managementPid + " verwaltet werden, aber es gibt keinen Konfigurationsverantwortlichen "
					               + "mit dieser Pid", managementObject);
				}
				_authorityPid2Requester.put(managementPid, foreignConfigRequester);
				return foreignConfigRequester;
			}
		}
		// Es ist ein Typ-Objekt, dass hier nicht bercksichtigt wird
		return null;
	}

	public String getElementsManagementPid(final MutableSet mutableSet) {
		String managementPid = "";
		if(mutableSet instanceof ConfigMutableSet) {
			ConfigMutableSet configMutableSet = (ConfigMutableSet)mutableSet;
			managementPid = configMutableSet.getElementsManagementPid();
		}

		if(managementPid.length() == 0) {
			final ConfigurationAuthority setAuthority = mutableSet.getConfigurationArea().getConfigurationAuthority();
			managementPid = setAuthority.getPid();
		}
		return managementPid;
	}

	public ForeignMutableCollectionProxy getForeignMutableCollectionProxy(
			final short internalSimVariant, final MutableCollection mutableCollection) {
		final ForeignConfigRequester foreignConfigRequester = getForeignConfigRequester(mutableCollection);
		if(foreignConfigRequester == null) return null;
		return foreignConfigRequester.getForeignMutableCollectionProxy(internalSimVariant, mutableCollection);
	}

	public ClientDavInterface getConnection() {
		return _connection;
	}

	public DataModel getConfiguration() {
		return _configuration;
	}

	public SystemObject getLocalAuthority() {
		return _localAuthority;
	}

	void start() {
		_executor = Executors.newSingleThreadScheduledExecutor();
		_foreignConfigReceiveCommunicator.start();
	}

	ScheduledExecutorService getExecutor() {
		return _executor;
	}

	void memorizeRemoteObject(final Long key, final ForeignDynamicObject remoteObject) {
		synchronized(_foreignObjects) {
			_foreignObjects.put(key, remoteObject);
		}
	}

	public SystemObject getRemoteObject(final Long key) {
		synchronized(_foreignObjects) {
			return _foreignObjects.get(key);
		}
	}

	public boolean hasRemoteObject(final Long key) {
		synchronized(_foreignObjects) {
			return _foreignObjects.containsKey(key);
		}
	}

	public void updateNotValidSince(final long objectId, final long objectTypeId, final long notValidSince) {
		final SystemObject remoteObject = getRemoteObject(objectId);
		if(remoteObject instanceof ForeignDynamicObject) {
			final ForeignDynamicObject foreignDynamicObject = ((ForeignDynamicObject)remoteObject);
			final SystemObjectType remoteObjectType = foreignDynamicObject.getType();
			if(remoteObjectType instanceof ConfigDynamicObjectType && remoteObjectType.getId() == objectTypeId) {
				foreignDynamicObject.setNotValidSince(notValidSince);
				final ConfigDynamicObjectType configDynamicObjectType = ((ConfigDynamicObjectType)remoteObjectType);
				configDynamicObjectType.informInvalidationListener(foreignDynamicObject);
			}
		}
	}

	public void updateName(final long objectId, final long objectTypeId, final String newName) {
		final SystemObject remoteObject = getRemoteObject(objectId);
		if(remoteObject instanceof ForeignDynamicObject) {
			final ForeignDynamicObject foreignDynamicObject = ((ForeignDynamicObject)remoteObject);
			final SystemObjectType remoteObjectType = foreignDynamicObject.getType();
			if(remoteObjectType instanceof ConfigDynamicObjectType && remoteObjectType.getId() == objectTypeId) {
				foreignDynamicObject.setName(newName);
				final ConfigDynamicObjectType configDynamicObjectType = ((ConfigDynamicObjectType)remoteObjectType);
				configDynamicObjectType.informNameChangedListener(foreignDynamicObject);
			}
		}
	}
}
