/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.lib.einstellungen;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenSpeicher;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenWaehler;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenWaehler.EinstellungsArt;
import de.inovat.buv.plugin.lib.Activator;
import de.inovat.buv.plugin.lib.debug.Log;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.PlatformUI;

import java.util.ArrayList;
import java.util.List;

//~ KLASSEN ===================================================================

/**
 * Klasse verwaltet die Speicherfunktionen.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 1.1 $ / $Date: 2010/05/07 16:03:36 $ / ($Author: Pittner $)
 */
public class SpeicherFunktionen {

    /**
     * Wird zu ID des zu speicherten Parameters addiert,
     * falls der Speicher synchronisiert werden soll.
     */
    private final static String ID_ZEITPUNKT = ".syncZeitPunkt";

    /**
     * Stellt fest, ob der SpeicherProzess gerade luft.
     * Whrend des SpeicherProzesses werden weitere Prozesse
     * (Speichern und Laden mit Synchronisation) gesperrt.
     */
    private static boolean speicherProzessLaeuft = false;

    //~ KONSTANTE AUFZAEHLUNGSTYPEN ===========================================

    /**
     * Aufzhlung fr die Art und den Ort vom Speicher.
     */
    public static enum ArtOrt { ALLGEMEIN_LOKAL, ALLGEMEIN_DAV, BENUTZER_LOKAL, BENUTZER_DAV }

    //~ METHODEN ==============================================================

    /**
     * Ermittle den aktuellsten ArtOrt.
     * Falls alle Speicher gleich aktuell sind,
     * wird null zurckgegeben.
     *
     * @param parameterID ID des Parameters
     * @param speicherArtOrt Definiert Art, Ort von Speicher.
     *
     * @return Aktuellster ArtOrt oder null, falls alle Speicher gleich aktuell sind.
     */
    private static ArtOrt ermittleAktuellsteSpeicherArt(String parameterID, SpeicherArtUndOrt speicherArtOrt) {

        // Lese die Aktualisierungszeitpunkte:
        long[] arZeitpunkt = new long[speicherArtOrt.getArArtOrt().length];

        for (int i = 0; i < arZeitpunkt.length; i++) {
            arZeitpunkt[i] = ermittleDatum(parameterID, speicherArtOrt.getArArtOrt()[i]);
        }

        // Berechne max. Aktualisierungszeitpunkt und zugehrigen Speicherart:
        boolean istDatumGleich = true;
        ArtOrt  aktArtOrt      = speicherArtOrt.getArArtOrt()[0];
        long    maxDatum       = arZeitpunkt[0];

        for (int i = 1; i < arZeitpunkt.length; i++) {
            if (maxDatum < arZeitpunkt[i]) {
                maxDatum       = arZeitpunkt[i];
                aktArtOrt      = speicherArtOrt.getArArtOrt()[i];
                istDatumGleich = false;
            } else if (maxDatum > arZeitpunkt[i]) {
                istDatumGleich = false;
            }
        }

        // Falls den Aktualisierungszeitpunkt berall gleich ist,
        // sind alle Speicher im aktuellen Zustand:
        if (istDatumGleich) {
            aktArtOrt = null;
        }

        return aktArtOrt;
    }

    /**
     * Ermittle die Beschreibung fr die Speicherart.
     *
     * @param artOrt Art und Ort von Speicher.
     *
     * @return Beschreibung fr die Speicherart.
     */
    private static String ermittleBeschreibung(ArtOrt artOrt) {
        switch (artOrt) {
        case BENUTZER_DAV :
            return "fr den Benutzer global";

        case BENUTZER_LOKAL :
            return "fr den Benutzer lokal";

        case ALLGEMEIN_DAV :
            return "allgemein global";

        case ALLGEMEIN_LOKAL :
            return "allgemein lokal";

        default :
            return "";
        }
    }

    /**
     * Ermittle den Aktualisierungszeitpunkt fr den Parameter und die Speicherart.
     *
     * @param parameterID ID des Parameters.
     * @param artOrt Definiert Art, Ort von Speicher.
     *
     * @return Aktualisierungszeitpunkt fr den Parameter und die Speicherart
     * oder 0.
     */
    private static long ermittleDatum(String parameterID, ArtOrt artOrt) {
        String                datumString = "0";
        EinstellungenSpeicher speicher    = ermittleSpeicher(artOrt);

        // Lese den Aktualisierungszeitpunkt aus:
        if (speicher != null) {
            datumString = speicher.getParameter(parameterID + ID_ZEITPUNKT);
        }

        long datum;

        // Konvertiere den Aktualisierungszeitpunkt in long:
        try {
            datum = Long.valueOf(datumString);
        }
        catch (NumberFormatException e) {
            datum = 0;
        }

        return datum;
    }

    /**
     * Ermittle EinstellungenSpeicher fr die Speicherart.
     *
     * @param artOrt Art und Ort von Speicher.
     *
     * @return EinstellungenSpeicher fr die Speicherart oder null.
     */
    private static EinstellungenSpeicher ermittleSpeicher(ArtOrt artOrt) {
        switch (artOrt) {
        case BENUTZER_DAV :
            return EinstellungenWaehler.getInstanz().getEinstellungen(EinstellungsArt.benutzerNetzweit);

        case BENUTZER_LOKAL :
            return EinstellungenWaehler.getInstanz().getEinstellungen(EinstellungsArt.benutzerLokal);

        case ALLGEMEIN_DAV :
            return EinstellungenWaehler.getInstanz().getEinstellungen(EinstellungsArt.allgemeinNetzweit);

        case ALLGEMEIN_LOKAL :
            return EinstellungenWaehler.getInstanz().getEinstellungen(EinstellungsArt.allgemeinLokal);

        default :
            return null;
        }
    }

    /**
     * Ermittle den gespeicherten Parameter.
     *
     * @param speicherArtOrt Definiert Art, Ort von Speicher.
     *
     * @return Gespeicherter Parameter.
     */
    public static String ladeEinstellung(SpeicherArtUndOrt speicherArtOrt) {
        return ladeParameter(speicherArtOrt.getParameterID(), speicherArtOrt);
    }

    /**
     * Ermittle den gespeicherten Parameter als Boolean.
     *
     * @param speicherArtOrt Definiert Art, Ort von Speicher.
     *
     * @return Parameterwert als Boolean oder null.
     */
    public static Boolean ladeEinstellungAlsBoolean(SpeicherArtUndOrt speicherArtOrt) {
        return ladeParameterAlsBoolean(speicherArtOrt.getParameterID(), speicherArtOrt);
    }

    /**
     * Ermittle den gespeicherten Parameter als Integer.
     *
     * @param speicherArtOrt Definiert Art, Ort von Speicher.
     *
     * @return Parameterwert als Integer oder null.
     */
    public static Integer ladeEinstellungAlsInteger(SpeicherArtUndOrt speicherArtOrt) {
        return ladeParameterAlsInteger(speicherArtOrt.getParameterID(), speicherArtOrt);
    }

    /**
     * Ermittle den gespeicherten Parameter.
     *
     * @param parameterID ID des Parameters, der ermittelt werden soll.
     * Muss eindeutig sein.
     *
     * @param speicherArtOrt Definiert Art, Ort von Speicher.
     *
     * @return Parameterwert oder null.
     */
    public static String ladeParameter(String parameterID, SpeicherArtUndOrt speicherArtOrt) {
        String parameter = null;

        if ((parameterID != null) || (speicherArtOrt != null)) {
            EinstellungenSpeicher speicher;

            if (speicherArtOrt.isSynchronisieren()) {

                // Warte bis der SpeicherProzess beendet wird:
                while (speicherProzessLaeuft) {
                    try {
                        Thread.sleep(100);
                    }
                    catch (InterruptedException e) {
                        Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
                    }
                }

                // Lade den Parameter mit der Synchronisation:
                ArtOrt  aktArtOrt         = ermittleAktuellsteSpeicherArt(parameterID, speicherArtOrt);
                boolean isSynchronisieren = true;

                // Falls alle Speicher gleich aktuell sind
                // (aktArtOrt==null), braucht man keine Synchronisation:
                if (aktArtOrt == null) {
                    aktArtOrt         = speicherArtOrt.getArArtOrt()[0];
                    isSynchronisieren = false;
                }

                speicher = ermittleSpeicher(aktArtOrt);

                if (speicher != null) {
                    parameter = speicher.getParameter(parameterID);

                    // Synchronisiere (Speichere Parameter fr alle Speicher aus dem Array auer aktuellen):
                    if (isSynchronisieren) {
                        speichereParameterOhneFehlerMeldung(parameterID, speicherArtOrt, parameter);
                    }
                }
            } else {

                // Lade den Parameter ohne die Synchronisation
                // (suche in aller Speicher, falls gefunden - unterbreche):
                for (ArtOrt artOrt : speicherArtOrt.getArArtOrt()) {
                    speicher = ermittleSpeicher(artOrt);

                    if (speicher != null) {
                        parameter = speicher.getParameter(parameterID);
                    }

                    if (parameter != null) {
                        break;
                    }
                }
            }
        }

        return parameter;
    }

    /**
     * Ermittle den gespeicherten Parameter als Boolean.
     *
     * @param parameterID ID des Parameters, der ermittelt werden soll.
     * Muss eindeutig sein.
     *
     * @param speicherArtOrt Definiert Art, Ort von Speicher.
     *
     * @return Parameterwert als Boolean oder null.
     */
    public static Boolean ladeParameterAlsBoolean(String parameterID, SpeicherArtUndOrt speicherArtOrt) {
        Boolean boolWert = null;
        String  text     = ladeParameter(parameterID, speicherArtOrt);

        if (text != null) {
            try {
                boolWert = Boolean.valueOf(text);
            }
            catch (Exception e) {
                boolWert = null;
            }
        }

        return boolWert;
    }

    /**
     * Ermittle den gespeicherten Parameter als Integer.
     *
     * @param parameterID ID des Parameters, der ermittelt werden soll.
     * Muss eindeutig sein.
     *
     * @param speicherArtOrt Definiert Art, Ort von Speicher.
     *
     * @return Parameterwert als Integer oder null.
     */
    public static Integer ladeParameterAlsInteger(String parameterID, SpeicherArtUndOrt speicherArtOrt) {
        Integer intWert = null;
        String  text    = ladeParameter(parameterID, speicherArtOrt);

        if (text != null) {
            try {
                intWert = Integer.valueOf(text);
            }
            catch (NumberFormatException e) {
                intWert = 0;
            }
        }

        return intWert;
    }

    /**
     * Fr alle Speicherarten lsche Parameter
     * und sichere alle Speicher. Falls die Variable _synchronisieren aus
     * speicherArtOrt auf true gesetzt ist, wird gleichzeitig den ZeitPunkt
     * mitgelscht.
     * Warnungen, in welchem Speicher der Parameter nicht gelscht werden kann,
     * werden nur dann ausgegeben, falls speicherArtOrt.isWarnungenAusgeben()
     * false zurck gibt.
     *
     * @param parameterID ID des zu lschenden Parameters.
     * Muss eindeutig sein.
     *
     * @param speicherArtOrt Art und Ort von Speicher.
     *
     * @return true, falls der Parameter erfolgreich (mind. einmal) gelscht wird,
     * false - sonst.
     */
    public static boolean loescheParameter(String parameterID, SpeicherArtUndOrt speicherArtOrt) {
        boolean       erfolgreich    = false;  // Definiert, ob der Parameter mind. ein Mal gelscht wird.
        List<IStatus> listeWarnungen = new ArrayList<IStatus>();

        if ((parameterID != null) && (speicherArtOrt != null)) {
            boolean               geloeschtEinzeln;
            EinstellungenSpeicher speicher;

            for (ArtOrt artOrt : speicherArtOrt.getArArtOrt()) {
                speicher = ermittleSpeicher(artOrt);

                // Speichere:
                if (speicherArtOrt.isSynchronisieren()) {

                    // Lsche mit der Synchronisation:
                    geloeschtEinzeln = ((speicher != null) && (speicher.loescheParameter(parameterID)) && (speicher.loescheParameter(parameterID + ID_ZEITPUNKT)) && (sichereSpeicher(artOrt)));
                } else {

                    // Lsche ohne die Synchronisation:
                    geloeschtEinzeln = ((speicher != null) && (speicher.loescheParameter(parameterID)) && (sichereSpeicher(artOrt)));
                }

                // Bewerte:
                if (geloeschtEinzeln) {
                    erfolgreich = true;
                } else {
                    listeWarnungen.add(Log.erzeugeMeldung(Log.WARNUNG, Activator.PLUGIN_ID, "Der Parameter <" + parameterID + "> kann nicht " + ermittleBeschreibung(artOrt) + " gelscht werden!"));
                }
            }

            // Meldungausgaben:
            if (speicherArtOrt.isWarnungenAusgeben()) {

                // Warnungen:
                for (IStatus meldung : listeWarnungen) {
                    Log.zeige(meldung);
                }
            }
        }

        return erfolgreich;
    }

    /**
     * Sichere (speichere alle seine Parameter) den Speicher.
     *
     * @param artOrt Art und Ort von Speicher.
     *
     * @return true, falls die Sicherung erfolgreich war,
     * false - sonst.
     */
    private static boolean sichereSpeicher(ArtOrt artOrt) {
        EinstellungenWaehler.getInstanz().setSendenOhneUrlasserDialog(true);

        switch (artOrt) {
        case BENUTZER_DAV :
            return EinstellungenWaehler.getInstanz().sichereSpeicherBenutzerNetzweit();

        // return EinstellungenWaehler.getInstanz().sichereEinstellungenSpeicherBenutzerBestimmtNetzweit("lg", ermittleSpeicher(BENUTZER_NETZ));
        case BENUTZER_LOKAL :
            EinstellungenWaehler.getInstanz().sichereSpeicherBenutzerLokal();

            return true;

        case ALLGEMEIN_DAV :
            return EinstellungenWaehler.getInstanz().sichereSpeicherAllgemeinNetzweit();

        case ALLGEMEIN_LOKAL :
            return EinstellungenWaehler.getInstanz().sichereSpeicherAllgemeinLokal();
        }

        EinstellungenWaehler.getInstanz().setSendenOhneUrlasserDialog(false);

        return false;
    }

    /**
     * Speichere den Wert.
     *
     * @param speicherArtOrt Definiert Art, Ort von Speicher und ID des zu speicherten Parameters.
     * @param wert Wert, das gespeichert werden soll.
     *
     * @return true, falls das Speichern erfolgreich war,
     * false - sonst.
     */
    public static boolean speichereEinstellung(SpeicherArtUndOrt speicherArtOrt, String wert) {
        return speichereParameter(speicherArtOrt.getParameterID(), speicherArtOrt, wert);
    }

    /**
     * Fr alle Speicherarten berschreibe Parameter mit dem neuen Wert
     * und sichere alle Speicher. Falls die Variable _synchronisieren aus
     * speicherArtOrt auf true gesetzt ist, wird gleichzeitig den ZeitPunkt
     * mitgespeichert.
     * Falls der Parameter nirgendwo gespeichert wird, wird die _Fehler-Meldung angezeigt.
     * Warnungen, in welchem Speicher der Parameter nicht gespeichert werden kann,
     * werden nur dann ausgegeben, falls speicherArtOrt.isWarnungenAusgeben()
     * false zurck gibt.
     *
     *
     * @param parameterID ID des berschreibenden Parameters.
     * Muss eindeutig sein.
     *
     * @param speicherArtOrt Art und Ort von Speicher.
     *
     * @param wert Neuer Wert fr den Parameter.
     *
     * @return true, falls der Parameter erfolgreich (mind. einmal) gespeichert wird,
     * false - sonst.
     */
    public static boolean speichereParameter(String parameterID, SpeicherArtUndOrt speicherArtOrt, String wert) {
        boolean erfolgreich = speichereParameterOhneFehlerMeldung(parameterID, speicherArtOrt, wert);  // Definiert, ob der Parameter mind. ein Mal gespeichert wird.

        if (!erfolgreich) {
	
            // _Fehler:
            MessageDialog.openError(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(), "Fehler", "ACHTUNG! Der Parameter <" + parameterID + "> wird nicht gespeichert!");
            Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "ACHTUNG! Der Parameter <" + parameterID + "> wird nicht gespeichert!"));
        }

        return erfolgreich;
    }

    /**
     * Fr alle Speicherarten berschreibe Parameter mit dem neuen Wert
     * und sichere alle Speicher. Falls die Variable _synchronisieren aus
     * speicherArtOrt auf true gesetzt ist, wird gleichzeitig den ZeitPunkt
     * mitgespeichert.
     * Warnungen, in welchem Speicher der Parameter nicht gespeichert werden kann,
     * werden nur dann ausgegeben, falls speicherArtOrt.isWarnungenAusgeben()
     * false zurck gibt.
     *
     * _WICHTIG: Whrend dieses Prozesses werden weitere Prozesse
     * (Speichern und Laden mit Synchronisation) gesperrt.
     *
     * @param parameterID ID des berschreibenden Parameters.
     * Muss eindeutig sein.
     *
     * @param speicherArtOrt Art und Ort von Speicher.
     *
     * @param wert Neuer Wert fr den Parameter.
     *
     * @return true, falls der Parameter erfolgreich (mind. einmal) gespeichert wird,
     * false - sonst.
     */
    private static boolean speichereParameterOhneFehlerMeldung(String parameterID, SpeicherArtUndOrt speicherArtOrt, String wert) {
        boolean       erfolgreich    = false;  // Definiert, ob der Parameter mind. ein Mal gespeichert wird.
        List<IStatus> listeWarnungen = new ArrayList<IStatus>();

        // Warte bis alter SpeicherProzess beendet wird:
        while (speicherProzessLaeuft) {
            try {
                Thread.sleep(100);
            }
            catch (InterruptedException e) {
                Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
            }
        }

        // Sperre weitere Prozesse (Speichern und Laden mit Synchronisation),
        // bis der Parameter vollstndig gespeichert wird:
        speicherProzessLaeuft = true;

        if ((parameterID != null) && (speicherArtOrt != null) && (wert != null)) {
            boolean               gespeichertEinzeln;
            EinstellungenSpeicher speicher;
            String                datum = "" + System.currentTimeMillis();

            for (ArtOrt artOrt : speicherArtOrt.getArArtOrt()) {
                speicher = ermittleSpeicher(artOrt);

                // Speichere:
                if (speicherArtOrt.isSynchronisieren()) {

                    // Speichere mit der Synchronisation (Datum):
                    gespeichertEinzeln = ((speicher != null) && (speicher.ueberschreibeParameter(parameterID, wert)) && (speicher.ueberschreibeParameter(parameterID + ID_ZEITPUNKT, datum)) && (sichereSpeicher(artOrt)));
                } else {

                    // Speichere ohne die Synchronisation:
                    gespeichertEinzeln = ((speicher != null) && (speicher.ueberschreibeParameter(parameterID, wert)) && (sichereSpeicher(artOrt)));
                }

                // Bewerte:
                if (gespeichertEinzeln) {
                    erfolgreich = true;
                } else {
                    listeWarnungen.add(Log.erzeugeMeldung(Log.WARNUNG, Activator.PLUGIN_ID, "Der Parameter <" + parameterID + "> kann nicht " + ermittleBeschreibung(artOrt) + " gespeichert werden!"));
                }
            }

            // Meldungausgaben:
            if (speicherArtOrt.isWarnungenAusgeben()) {

                // Warnungen:
                for (IStatus meldung : listeWarnungen) {
                    Log.zeige(meldung);
                }
            }
        }

        // Freigabe:
        speicherProzessLaeuft = false;

        return erfolgreich;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 09.03.10
